# Copyright (c) 2010, Marco Gallotta
# Released under terms of the MIT/X/Expat Licence. See COPYING for details.

from datetime import timedelta
from operator import itemgetter
from urllib import quote

import ibid
from ibid.compat import dt_strptime, ElementTree
from ibid.config import BoolOption
from ibid.plugins import Processor, match
from ibid.utils import ago, generic_webservice, human_join, indefinite_article

features = {}

experience_requirements = [
    0, 2, 4, 6, 8, 10, 15, 20, 30, 40, 50, 60, 70, 80, 100, 125, 150, 200,
    300, 500, 1000, 1500, 2000, 3000, 5000, 7000, 10000, 15000, 20000, 25000
]
experience_requirements = [(points, str(level+1)) for level, points in
    enumerate(experience_requirements)]
work_skill_requirements = [
    (0,     u'Apprentice'),
    (20,    u'Assistant'),
    (100,   u'Junior'),
    (500,   u'Senior'),
    (2000,  u'Coordinator'),
    (5000,  u'Specialist'),
    (10000, u'Expert'),
    (20000, u'Master'),
    (40000, u'Guru'),
]
mil_skill_requirements = [
    (0,      u'Greenhorn'),
    (40,     u'Rookie'),
    (200,    u'Hotshot'),
    (1000,   u'Marksman'),
    (4000,   u'Sharp Shooter'),
    (10000,  u'Professional'),
    (20000,  u'Expert'),
    (40000,  u'Ranger'),
    (80000,  u'Nemesis'),
    (160000, u'Veteran'),
]
rank_requirements = [
    (0,     u'Private'),
    (51,    u'Corporal'),
    (151,   u'Sergeant'),
    (301,   u'Lieutenant'),
    (601,   u'Captain'),
    (1201,  u'Colonel'),
    (2501,  u'General'),
    (5001,  u'Field Marshall'),
]

def skill_map(name):
    # Fix API fail
    if name == 'Commodities Worker':
        return 'Harvester'
    return name

def skill_points_to_level(points, skill_requirements):
    level, name, next = 0, 0, 0
    for required, skill_name in skill_requirements:
        if points >= required:
            name = skill_name
            level += 1
        else:
            next = required
            break
    if points >= required:
        required = skill_requirements[-1][0]*2
        if skill_requirements is rank_requirements:
            required = skill_requirements[-1][0] + 5000
        next = required
        while points >= required:
            name += '*'
            if skill_requirements is rank_requirements:
                required += 5000
            else:
                required *= 2
            level += 1
            next = required
    return (level, name, next)

def experience_level(points):
    return skill_points_to_level(points, experience_requirements)

def work_skill_points_to_level(points):
    return skill_points_to_level(points, work_skill_requirements)

def mil_skill_points_to_level(points):
    return skill_points_to_level(points, mil_skill_requirements)

def points_to_rank(points):
    return skill_points_to_level(points, rank_requirements)

features['erepublik'] = {
    'description': u'Lookup data from the eRepublik game server',
    'categories': ('fun', 'game', 'lookup',),
}
class Citizen(Processor):
    usage = u"""Lookup citizen data
    erepublik link <user|id>
    erepublik citizen <user|id>
    erepublik work <user|id>
    erepublik military <user|id>"""

    features = ('erepublik',)
    require_prefix = BoolOption('require_prefix',
            "Require 'erepublik' prefix in commands", True)

    def citizen_tree(self, name):
        base_url = u'http://api.erepublik.com/v2/feeds/citizen'
        try:
            id = int(name)
            url = u'%ss/%d' % (base_url, id)
        except ValueError:
            url = u'%s_by_name/xml/%s' % (base_url, quote(name))
        data = generic_webservice(url)
        tree = ElementTree.fromstring(data)
        return tree

    def get_skills(self, tree, type):
        skills = {}
        for skill in tree.find(
                {'work': 'skills', 'military': 'military-skills'}[type]) \
                .getiterator(
                {'work': 'skill', 'military': 'military-skill'}[type]):
            skills[skill.findtext('./name')] = int(skill.findtext('./points'))
        return skills

    def get_age(self, tree, now):
        age = now - dt_strptime(tree.findtext('./date-of-birth'),
            '%Y-%m-%dT%H:%M:%SZ') - timedelta(hours=9)
        return ago(age, 2)

    def get_experience(self, tree):
        return int(tree.findtext('./experience-points'))

    def get_rank(self, tree):
        return int(tree.findtext('./military/rank-points'))

    def best_skill(self, skills):
        best = (None, -1)
        for name, points in skills.items():
            if points > best[1]:
                best = (name, points)
        return best[0]

    def skill_text(self, tree, type, skill=None, article=False):
        skill_function = {
            'experience': experience_level,
            'work': work_skill_points_to_level,
            'military': mil_skill_points_to_level,
            'rank': points_to_rank,
        }[type]
        if type == 'experience':
            points = self.get_experience(tree)
        elif type == 'rank':
            points = self.get_rank(tree)
        else:
            points = self.get_skills(tree, type)[skill]
        level, name, required = skill_function(points)

        if type in ('experience', 'rank'):
            full_name = name
        else:
            full_name = '%s %s' % (name, skill_map(skill))

        art = ''
        if article:
            art = indefinite_article(full_name) + ' '

        return u'%(article)s%(name)s (%(points)s/%(required)d)' % {
            'article': art,
            'name': full_name.lower(),
            'points': points,
            'required': required,
        }

    def get_medals(self, tree):
        medals = []
        for medal in tree.find('./medals').getiterator('medal'):
            type = medal.findtext('type')
            amount = int(medal.findtext('amount'))
            if amount > 0:
                medals.append(u'%d %s' % (amount, type))
        if medals:
            return u'%s medals' % human_join(medals)
        else:
            return u'no medals'

    @match(r'^(erep(?:ublik)?\s+)?link\s+(.+)$')
    def link(self, event, prefix, name):
        if not prefix and self.require_prefix:
            return
        tree = self.citizen_tree(name)
        event.addresponse(u'http://www.erepublik.com/en/citizen/profile/%s',
            tree.findtext('./id'))

    @match(r'^(erep(?:ublik)?\s+)?cit(?:izen)?\s+(.+)$')
    def citizen(self, event, prefix, name):
        if not prefix and self.require_prefix:
            return
        tree = self.citizen_tree(name)

        politician = ''
        if tree.findtext('./is-president') == 'true':
            politician = 'the president '
        elif tree.findtext('./is-congressman') == 'true':
            politician = 'a congressman '
        skills = self.get_skills(tree, 'work')
        mil_skills = self.get_skills(tree, 'military')

        event.addresponse(u'%(name)s is %(politician)sin %(region)s, '
            u'%(residency)s with '
            u'citizenship of %(citizenship)s, has wellness %(wellness)s and '
            u'happiness %(happiness)s, is on level %(experience)s, is '
            u'%(work_skill)s and %(mil_skill)s with rank %(rank)s and has '
            u'%(medals)s after %(age)s', {
                'name': tree.findtext('./name'),
                'politician': politician,
                'region': tree.findtext('./residence/region/name'),
                'residency': tree.findtext('./residence/country/name'),
                'citizenship': tree.findtext('./citizenship/country/name'),
                'wellness': tree.findtext('./wellness'),
                'happiness': tree.findtext('./happiness'),
                'experience': self.skill_text(tree, 'experience'),
                'work_skill': self.skill_text(tree, 'work',
                    self.best_skill(skills), article=True),
                'mil_skill': self.skill_text(tree, 'military',
                    self.best_skill(mil_skills), article=True),
                'rank': self.skill_text(tree, 'rank'),
                'medals': self.get_medals(tree),
                'age': self.get_age(tree, event.time),
            })

    @match(r'^(erep(?:ublik)?\s+)?(work|military)\s+(.+)$')
    def skills(self, event, prefix, type, name):
        if not prefix and self.require_prefix:
            return
        tree = self.citizen_tree(name)

        skills = self.get_skills(tree, type)
        skills_text = []
        for skill, points in sorted(skills.items(), key=itemgetter(1),
                reverse=True):
            if points > 0:
                skills_text.append(self.skill_text(tree, type, skill))

        if skills_text:
            event.addresponse(u'%(name)s is %(article)s %(skills)s', {
                'name': tree.findtext('./name'),
                'article': indefinite_article(skills_text[0]),
                'skills': human_join(skills_text),
            })
        else:
            event.addresponse(u'%(name)s is stupid, he knows nothing!',
                {
                'name': tree.findtext('./name'),
            })

class Battle(Processor):
    usage = """Lookup erpublik battle info
    erepublik battle <id>"""

    features = ('erepublik',)
    require_prefix = BoolOption('require_prefix',
            "Require 'erepublik' prefix in commands", True)

    def battle_tree(self, event, id):
        url = u'http://api.erepublik.com/v2/feeds/battles/%d' % id
        data = generic_webservice(url)
        tree = ElementTree.fromstring(data)
        if tree.tag == 'error':
            event.addresponse(u"That battle doesn't exist yet!")
            return None
        return tree

    def battle_text(self, tree, now):
        battle = u'%(attacker)s attacked %(region)s, %(defender)s' % {
            'attacker': tree.findtext('./attacker/name'),
            'region': tree.findtext('./region/name'),
            'defender': tree.findtext('./defender/name'),
        }
        if not tree.findtext('./progress/finished-at'):
            age = now - dt_strptime(tree.findtext('./progress/started-at')[:-6],
                '%Y-%m-%dT%H:%M:%S')
            battle += u' %s ago' % ago(age, 2)
        else:
            reason = tree.findtext('./progress/finished-reason')
            if reason == 'RETREAT':
                result = u'%s retreated' % tree.findtext('./attacker/name')
            else:
                result = u'unimplemented reason'
            age = now - dt_strptime(tree.findtext('./progress/finished-at')[:-6],
                '%Y-%m-%dT%H:%M:%S')
            battle += u' and %s %s ago' % (result, ago(age, 2))
        return battle

    @match(r'^(erep(?:ublik)?\s+)?battle\s+(\d+)$')
    def battle(self, event, prefix, id):
        if not prefix and self.require_prefix:
            return
        tree = self.battle_tree(event, int(id))
        if not tree:
            return
        event.addresponse(self.battle_text(tree, event.time))


# vi: set et sta sw=4 ts=4:
